@@ -1,5 +1,5 @@
 /*
- * The IKE Scanner (ike-scan) is Copyright (C) 2003-2005 Roy Hills,
+ * The IKE Scanner (ike-scan) is Copyright (C) 2003-2007 Roy Hills,
  * NTA Monitor Ltd.
  *
  * This program is free software; you can redistribute it and/or
@@ -16,13 +16,24 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
+ * In addition, as a special exception, the copyright holders give
+ * permission to link the code of portions of this program with the
+ * OpenSSL library, and distribute linked combinations including the two.
+ *
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * file(s) with this exception, you may extend this exception to your
+ * version of the file(s), but you are not obligated to do so.  If you
+ * do not wish to do so, delete this exception statement from your
+ * version.
+ *
  * If this license is unacceptable to you, I may be willing to negotiate
  * alternative licenses (contact ike-scan@nta-monitor.com).
  *
  * You are encouraged to send comments, improvements or suggestions to
  * me at ike-scan@nta-monitor.com.
  *
- * $Id: utils.c,v 1.11 2005/01/21 13:36:06 rsh Exp $
+ * $Id: utils.c 9884 2007-01-14 19:05:39Z rsh $
  *
  * Author: Roy Hills
  * Date: 5 April 2004
@@ -30,12 +41,12 @@
  * This file contains various utility functions used by ike-scan.
  *
  * These functions were originally in ike-scan.c, but were moved to utils.c
- * because ike-scan.c was gettign rather large.
+ * because ike-scan.c was getting rather large.
  */
 
 #include "ike-scan.h"
 
-static char rcsid[] = "$Id: utils.c,v 1.11 2005/01/21 13:36:06 rsh Exp $";	/* RCS ID for ident(1) */
+static char rcsid[] = "$Id: utils.c 9884 2007-01-14 19:05:39Z rsh $";	/* RCS ID for ident(1) */
 
 
 /*
@@ -53,24 +64,29 @@ static char rcsid[] = "$Id: utils.c,v 1.11 2005/01/21 13:36:06 rsh Exp $";	/* RC
  *	None.
  */
 void
-timeval_diff(struct timeval *a, struct timeval *b, struct timeval *diff) {
+timeval_diff(const struct timeval *a, const struct timeval *b,
+             struct timeval *diff) {
+   struct timeval temp;
+
+   temp.tv_sec = b->tv_sec;
+   temp.tv_usec = b->tv_usec;
 
    /* Perform the carry for the later subtraction by updating b. */
-   if (a->tv_usec < b->tv_usec) {
-     int nsec = (b->tv_usec - a->tv_usec) / 1000000 + 1;
-     b->tv_usec -= 1000000 * nsec;
-     b->tv_sec += nsec;
+   if (a->tv_usec < temp.tv_usec) {
+     int nsec = (temp.tv_usec - a->tv_usec) / 1000000 + 1;
+     temp.tv_usec -= 1000000 * nsec;
+     temp.tv_sec += nsec;
    }
-   if (a->tv_usec - b->tv_usec > 1000000) {
-     int nsec = (a->tv_usec - b->tv_usec) / 1000000;
-     b->tv_usec += 1000000 * nsec;
-     b->tv_sec -= nsec;
+   if (a->tv_usec - temp.tv_usec > 1000000) {
+     int nsec = (a->tv_usec - temp.tv_usec) / 1000000;
+     temp.tv_usec += 1000000 * nsec;
+     temp.tv_sec -= nsec;
    }
  
    /* Compute the time difference
       tv_usec is certainly positive. */
-   diff->tv_sec = a->tv_sec - b->tv_sec;
-   diff->tv_usec = a->tv_usec - b->tv_usec;
+   diff->tv_sec = a->tv_sec - temp.tv_sec;
+   diff->tv_usec = a->tv_usec - temp.tv_usec;
 }
 
 /*
@@ -148,7 +164,7 @@ hstr_i(const char *cptr)
  *
  *	The returned pointer points to malloc'ed storage which should be
  *	free'ed by the caller when it's no longer needed.  If the length of
- *	the inputs string is not even, the function will return NULL and
+ *	the input string is not even, the function will return NULL and
  *	set data_len to 0.
  */
 unsigned char *
@@ -216,6 +232,57 @@ hex_or_str(const char *string, size_t *data_len) {
    }
 }
 
+/*
+ *	hex_or_num -- Convert hex or number to binary data
+ *
+ *	Inputs:
+ *
+ *	string		The hex or string to convert
+ *	data_len	(output) The length of the resultant binary data
+ *
+ *	Returns:
+ *
+ *	Pointer to the binary data, or NULL if an error occurred.
+ *
+ *	The input string must be in one of the following two formats:
+ *
+ *	0x<hex-data>	Input is in hex format
+ *	decimal number	Input is in numeric format
+ *
+ *	For numeric input, the binary data will be a 32-bit value in
+ *	big endian format.
+ *
+ *	The returned pointer points to malloc'ed storage which should be
+ *	free'ed by the caller when it's no longer needed.  If the length of
+ *	the inputs string is not even, the function will return NULL and
+ *	set data_len to 0.
+ */
+unsigned char *
+hex_or_num(const char *string, size_t *data_len) {
+
+   if (strlen(string) < 1) {	/* Input string too short */
+      *data_len = 0;
+      return NULL;
+   }
+
+   if (string[0] == '0' && string[1] == 'x') {	/* Hex input format */
+      return hex2data((string+2), data_len);
+   } else {					/* Assume number input format */
+      unsigned char *data;
+      size_t len = 4;	/* 32-bit value */
+      unsigned long value;
+      unsigned long value_be;
+
+      value = Strtoul(string, 10);
+      value_be = htonl(value);
+      data = Malloc(len);
+      memcpy(data, &value_be, len);
+      
+      *data_len = len;
+      return data;
+   }
+}
+
 /*
  * make_message -- allocate a sufficiently large string and print into it.
  *
@@ -300,10 +367,10 @@ numstr(unsigned num) {
  *	free'ed by the caller when it's no longer needed.
  */
 char *
-printable(unsigned char *string, size_t size) {
+printable(const unsigned char *string, size_t size) {
    char *result;
    char *r;
-   unsigned char *cp;
+   const unsigned char *cp;
    size_t outlen;
    unsigned i;
 /*
@@ -318,7 +385,7 @@ printable(unsigned char *string, size_t size) {
  *	Determine required size of output string.
  */
    if (!size)
-      size = strlen((char *) string);
+      size = strlen((const char *) string);
 
    outlen = size;
    cp = string;
@@ -412,10 +479,10 @@ printable(unsigned char *string, size_t size) {
  *	free'ed by the caller when it's no longer needed.
  */
 char *
-hexstring(unsigned char *data, size_t size) {
+hexstring(const unsigned char *data, size_t size) {
    char *result;
    char *r;
-   unsigned char *cp;
+   const unsigned char *cp;
    unsigned i;
 /*
  *	If the input data is NULL, return an empty string.
@@ -486,307 +553,6 @@ print_times(void) {
    time_last.tv_usec = time_now.tv_usec;
 }
 
-#ifndef HAVE_OPENSSL
-/*
- *	MD5 -- Calculate MD5 hash of specified data
- *
- *	Inputs:
- *
- *	d	The data to hash
- *	n	The length of the data
- *	md	The resulting MD5 hash
- *
- *	Returns:
- *
- *	The MD5 hash.
- *
- *	This function is a wrapper for the MD5 routines in md5.c.  If ike-scan
- *	was compiled with OpenSSL, then the OpenSSL MD5 routines are used
- *	instead, and this wrapper is not used.
- */
-unsigned char *
-MD5(const unsigned char *d, size_t n, unsigned char *md) {
-   md5_state_t context;
-   static unsigned char m[16];
-
-   if (md == NULL)	/* Use static storage if no buffer specified */
-      md=m;
-
-   md5_init(&context);
-   md5_append(&context, d, n);
-   md5_finish(&context, md);
-
-   return md;
-}
-#endif
-
-#ifndef HAVE_OPENSSL
-/*
- *	SHA1 -- Calculate SHA1 hash of specified data
- *
- *	Inputs:
- *
- *	d	The data to hash
- *	n	The length of the data
- *	md	The resulting SHA1 hash
- *
- *	Returns:
- *
- *	The SHA1 hash.
- *
- *	This function is a wrapper for the SHA1 routines in sha1.c.  If ike-scan
- *	was compiled with OpenSSL, then the OpenSSL MD5 routines are used
- *	instead, and this wrapper is not used.
- */
-unsigned char *
-SHA1(const unsigned char *d, size_t n, unsigned char *md) {
-   SHA1_CTX context;
-   static unsigned char m[20];
-
-   if (md == NULL)	/* Use static storage if no buffer specified */
-      md=m;
-
-   SHA1Init(&context);
-/*
- * SHA1Update's prototype doesn't use "const", so we use a cast to prevent
- * a warning.  It would really be better to fix sha1.[ch] so that they use
- * const, and I may do that some day.
- */
-   SHA1Update(&context, (unsigned char *)d, n);
-   SHA1Final(md, &context);
-
-   return md;
-}
-#endif
-
-/*
- *	hmac_md5 -- Calculate HMAC-MD5 keyed hash
- *
- *	Inputs:
- *
- *	text		The data to hash
- *	text_len	Length of the data in bytes
- *	key		The key
- *	key_len		Length of the key in bytes
- *	digest		The resulting HMAC-MD5 digest
- *
- *	Returns:
- *
- *	The HMAC-MD5 hash.
- *
- *	This function is based on the code from the RFC 2104 appendix.
- *
- *	We use #ifdef to select either the OpenSSL MD5 functions or the
- *	built-in MD5 functions depending on whether HAVE_OPENSSL is defined.
- *	This is faster that calling OpenSSL "HMAC" directly.
- */
-unsigned char *
-hmac_md5(const unsigned char *text, size_t text_len, const unsigned char *key,
-         size_t key_len, unsigned char *md) {
-   static unsigned char m[16];
-#ifdef HAVE_OPENSSL
-   MD5_CTX context;
-#else
-   md5_state_t context;
-#endif
-   unsigned char k_ipad[65];	/* inner padding -  key XORd with ipad */
-   unsigned char k_opad[65];    /* outer padding -  key XORd with opad */
-   unsigned char tk[16];
-   int i;
-
-   if (md == NULL)	/* Use static storage if no buffer specified */
-      md=m;
-
-   /* if key is longer than 64 bytes reset it to key=MD5(key) */
-   if (key_len > 64) {
-#ifdef HAVE_OPENSSL
-      MD5_CTX tctx;
-
-      MD5_Init(&tctx);
-      MD5_Update(&tctx, key, key_len);
-      MD5_Final(tk, &tctx);
-#else
-      md5_state_t tctx;
-
-      md5_init(&tctx);
-      md5_append(&tctx, key, key_len);
-      md5_finish(&tctx, tk);
-#endif
-
-      key = tk;
-      key_len = 16;
-   }
-   /*
-    * the HMAC_MD5 transform looks like:
-    *
-    * MD5(K XOR opad, MD5(K XOR ipad, text))
-    *
-    * where K is an n byte key
-    * ipad is the byte 0x36 repeated 64 times
-    * opad is the byte 0x5c repeated 64 times
-    * and text is the data being protected
-    */
-
-   /* start out by storing key in pads */
-   memset(k_ipad, '\0', sizeof k_ipad);
-   memset(k_opad, '\0', sizeof k_opad);
-   memcpy(k_ipad, key, key_len);
-   memcpy(k_opad, key, key_len);
-
-   /* XOR key with ipad and opad values */
-   for (i=0; i<64; i++) {
-      k_ipad[i] ^= 0x36;
-      k_opad[i] ^= 0x5c;
-   }
-#ifdef HAVE_OPENSSL
-   /*
-    * perform inner MD5
-    */
-   MD5_Init(&context);			/* init context for 1st pass */
-   MD5_Update(&context, k_ipad, 64);	/* start with inner pad */
-   MD5_Update(&context, text, text_len); /* then text of datagram */
-   MD5_Final(md, &context);		/* finish up 1st pass */
-   /*
-    * perform outer MD5
-    */
-   MD5_Init(&context);			/* init context for 2nd pass */
-   MD5_Update(&context, k_opad, 64);	/* start with outer pad */
-   MD5_Update(&context, md, 16);	/* then results of 1st hash */
-   MD5_Final(md, &context);		/* finish up 2nd pass */
-#else
-   /*
-    * perform inner MD5
-    */
-   md5_init(&context);			/* init context for 1st pass */
-   md5_append(&context, k_ipad, 64);	/* start with inner pad */
-   md5_append(&context, text, text_len); /* then text of datagram */
-   md5_finish(&context, md);		/* finish up 1st pass */
-   /*
-    * perform outer MD5
-    */
-   md5_init(&context);			/* init context for 2nd pass */
-   md5_append(&context, k_opad, 64);	/* start with outer pad */
-   md5_append(&context, md, 16);	/* then results of 1st hash */
-   md5_finish(&context, md);		/* finish up 2nd pass */
-#endif
-
-   return md;
-}
-
-/*
- *	hmac_sha1 -- Calculate HMAC-SHA1 keyed hash
- *
- *	Inputs:
- *
- *	text		The data to hash
- *	text_len	Length of the data in bytes
- *	key		The key
- *	key_len		Length of the key in bytes
- *	digest		The resulting HMAC-SHA1 digest
- *
- *	Returns:
- *
- *	The HMAC-SHA1 hash.
- *
- *	This function is based on the code from the RFC 2104 appendix.
- *
- *	We use #ifdef to select either the OpenSSL SHA1 functions or the
- *	built-in SHA1 functions depending on whether HAVE_OPENSSL is defined.
- *	This is faster that calling OpenSSL "HMAC" directly.
- */
-unsigned char *
-hmac_sha1(const unsigned char *text, size_t text_len, const unsigned char *key,
-          size_t key_len, unsigned char *md) {
-   static unsigned char m[20];
-#ifdef HAVE_OPENSSL
-   SHA_CTX context;
-#else
-   SHA1_CTX context;
-#endif
-   unsigned char k_ipad[65];	/* inner padding -  key XORd with ipad */
-   unsigned char k_opad[65];    /* outer padding -  key XORd with opad */
-   unsigned char tk[20];
-   int i;
-
-   if (md == NULL)	/* Use static storage if no buffer specified */
-      md=m;
-
-   /* if key is longer than 64 bytes reset it to key=SHA1(key) */
-   if (key_len > 64) {
-#ifdef HAVE_OPENSSL
-      SHA_CTX tctx;
-
-      SHA1_Init(&tctx);
-      SHA1_Update(&tctx, key, key_len);
-      SHA1_Final(tk, &tctx);
-#else
-      SHA1_CTX tctx;
-
-      SHA1Init(&tctx);
-      SHA1Update(&tctx, (unsigned char *)key, key_len);
-      SHA1Final(tk, &tctx);
-#endif
-
-      key = tk;
-      key_len = 20;
-   }
-   /*
-    * the HMAC_SHA1 transform looks like:
-    *
-    * SHA1(K XOR opad, SHA1(K XOR ipad, text))
-    *
-    * where K is an n byte key
-    * ipad is the byte 0x36 repeated 64 times
-    * opad is the byte 0x5c repeated 64 times
-    * and text is the data being protected
-    */
-
-   /* start out by storing key in pads */
-   memset(k_ipad, '\0', sizeof k_ipad);
-   memset(k_opad, '\0', sizeof k_opad);
-   memcpy(k_ipad, key, key_len);
-   memcpy(k_opad, key, key_len);
-
-   /* XOR key with ipad and opad values */
-   for (i=0; i<64; i++) {
-      k_ipad[i] ^= 0x36;
-      k_opad[i] ^= 0x5c;
-   }
-#ifdef HAVE_OPENSSL
-   /*
-    * perform inner SHA1
-    */
-   SHA1_Init(&context);			/* init context for 1st pass */
-   SHA1_Update(&context, k_ipad, 64);	/* start with inner pad */
-   SHA1_Update(&context, text, text_len); /* then text of datagram */
-   SHA1_Final(md, &context);		/* finish up 1st pass */
-   /*
-    * perform outer SHA1
-    */
-   SHA1_Init(&context);			/* init context for 2nd pass */
-   SHA1_Update(&context, k_opad, 64);	/* start with outer pad */
-   SHA1_Update(&context, md, 20);	/* then results of 1st hash */
-   SHA1_Final(md, &context);		/* finish up 2nd pass */
-#else
-   /*
-    * perform inner SHA1
-    */
-   SHA1Init(&context);			/* init context for 1st pass */
-   SHA1Update(&context, k_ipad, 64);	/* start with inner pad */
-   SHA1Update(&context, (unsigned char *)text, text_len); /* then text of datagram */
-   SHA1Final(md, &context);		/* finish up 1st pass */
-   /*
-    * perform outer SHA1
-    */
-   SHA1Init(&context);			/* init context for 2nd pass */
-   SHA1Update(&context, k_opad, 64);	/* start with outer pad */
-   SHA1Update(&context, md, 20);	/* then results of 1st hash */
-   SHA1Final(md, &context);		/* finish up 2nd pass */
-#endif
-
-   return md;
-}
-
 /*
  *	sig_alarm -- Signal handler for SIGALRM
  *
@@ -824,7 +590,8 @@ void sig_alarm(int signo) {
  *	but could be time consuming if the map contains a large number of
  *	entries.
  */
-char *id_to_name(int id, const id_name_map map[]) {
+const char *
+id_to_name(unsigned id, const id_name_map map[]) {
    int found = 0;
    int i = 0;
 
@@ -845,6 +612,202 @@ char *id_to_name(int id, const id_name_map map[]) {
       return numstr(id);
 }
 
+/*
+ *	name_to_id -- Return id associated with given name
+ *
+ *	Inputs:
+ *
+ *	name		The name to find in the map
+ *	id_name_map	Pointer to the id-to-name map
+ *
+ *	Returns:
+ *
+ *	The id associated with the name if an association is found in the
+ *	map, otherwise -1.
+ *
+ *	This function uses a sequential search through the map to find the
+ *	ID and associated name.  This is OK when the map is relatively small,
+ *	but could be time consuming if the map contains a large number of
+ *	entries.
+ *
+ *	The search is case-blind.
+ */
+int
+name_to_id(const char *name, const id_name_map map[]) {
+   int found = 0;
+   int i = 0;
+
+   if (map == NULL)
+      return -1;
+
+   while (map[i].id != -1) {
+      if ((str_ccmp(name,map[i].name)) == 0) {
+         found = 1;
+         break;
+      }
+      i++;
+   }
+
+   if (found)
+      return map[i].id;
+   else
+      return -1;
+}
+
+/* Standard BSD internet checksum routine */
+uint16_t
+in_cksum(uint16_t *ptr, size_t nbytes) {
+
+   register uint32_t sum;
+   uint16_t oddbyte;
+   register uint16_t answer;
+
+/*
+ * Our algorithm is simple, using a 32-bit accumulator (sum),
+ * we add sequential 16-bit words to it, and at the end, fold back
+ * all the carry bits from the top 16 bits into the lower 16 bits.
+ */
+
+   sum = 0;
+   while (nbytes > 1)  {
+      sum += *ptr++;
+      nbytes -= 2;
+   }
+
+/* mop up an odd byte, if necessary */
+   if (nbytes == 1) {
+      oddbyte = 0;            /* make sure top half is zero */
+      *((u_char *) &oddbyte) = *(u_char *)ptr;   /* one byte only */
+      sum += oddbyte;
+   }
+
+/*
+ * Add back carry outs from top 16 bits to low 16 bits.
+ */
+
+   sum  = (sum >> 16) + (sum & 0xffff);    /* add high-16 to low-16 */
+   sum += (sum >> 16);                     /* add carry */
+   answer = ~sum;          /* ones-complement, then truncate to 16 bits */
+   return(answer);
+}
+
+/*
+ *	random_byte -- Return a random byte in range 0..255
+ *
+ *	Inputs:	None
+ *
+ *	Returns: The random byte
+ */
+uint8_t
+random_byte(void) {
+   static union {
+      uint32_t longword;
+      uint8_t byte[4];
+   } random_data;
+   static int num_bytes = 0;	/* Number of bytes available */
+
+   if (num_bytes == 0) {
+      uint32_t random_value;
+
+      random_value = genrand_int32();
+      random_data.longword = htonl(random_value);
+      num_bytes = 4;
+   }
+   return random_data.byte[--num_bytes];
+}
+
+/*
+ *	random_ip	-- Return a random IP address
+ *
+ *	Imputs:	None
+ *	Returns: A random IP address
+ *
+ *	This returns a random IP address as a 32-bit value in host byte
+ *	order.
+ *
+ *	It will not return the following IP address ranges, because they
+ *	are invalid:
+ *
+ *	0/8, 1/8 or 2/8 - IANA reserved
+ *	127/8 - Localhost
+ *	Class D (Multicast)
+ *	Class E (Reserved)
+ */
+uint32_t
+random_ip(void) {
+   uint32_t random_value;
+   int acceptable;
+
+   do {
+      random_value = genrand_int32();
+      if ((random_value & 0xff000000) == 0x7f000000 ||	/* 127.x.x.x */
+          random_value > 0xefffffff ||			/* Class D or E */
+          random_value < 0x03000000) {			/* 0/8, 1/8 or 2/8 */
+         acceptable = 0;
+      } else {
+         acceptable = 1;
+      }
+   } while (!acceptable);
+
+   return random_value;
+}
+
+/*
+ *	str_ccmp  -- Case-blind string comparison
+ *
+ *	Inputs:
+ *
+ *	s1 -- The first input string
+ *	s2 -- The second input string
+ *
+ *	Returns:
+ *
+ *	An integer indicating whether s1 is less than (-1), the same as (0),
+ *	or greater than (1) s2.
+ *
+ *	This function performs the same function, and takes the same arguments
+ *	as the common library function strcasecmp.  This function is used
+ *	instead because strcasecmp is not portable.
+ */
+int
+str_ccmp( const char *s1, const char *s2 ) {
+   int c1, c2;
+
+   for( ; ; s1++, s2++ ){
+      c1 = tolower( (unsigned char) *s1 );
+      c2 = tolower( (unsigned char) *s2 );
+
+      if( c1 > c2            )  return   1;
+      if( c1 < c2            )  return  -1;
+      if( c1 == 0 && c2 == 0 )  return   0;
+   }
+} 
+
+/*
+ *	name_or_number -- Calculate the numeric value of a string containing
+ *	                  either a name from a map, or a number.
+ *
+ *	Inputs:
+ *
+ *	string		The input string
+ *	map		The ID/name map
+ */
+unsigned
+name_or_number(const char *string, const id_name_map map[]) {
+   int result;
+   char *endptr;
+
+   result=strtoul(string, &endptr, 0);
+   if (endptr != string)  /* Successful conversion */
+      return result;
+
+   result=name_to_id(string, map);
+   if (result == -1)
+      err_msg("Invalid value: %s", string);
+
+   return result;
+}
+
 void utils_use_rcsid(void) {
    fprintf(stderr, "%s\n", rcsid);	/* Use rcsid to stop compiler optimising away */
 }
